@@ -10,12 +10,18 @@ from lintian_brush.control import (
     ensure_minimum_version,
     get_relation,
     read_debian_compat_file,
-    update_control,
+    ControlUpdater,
     )
 from lintian_brush.debhelper import (
     detect_debhelper_buildsystem,
-    DEBHELPER_BUILD_STEPS,
     lowest_non_deprecated_compat_level,
+    maximum_debhelper_compat_version,
+    DEBHELPER_BUILD_STEPS,
+    )
+from lintian_brush.fixer import (
+    compat_release,
+    current_package_version,
+    report_result,
     )
 from lintian_brush.rules import (
     check_cdbs,
@@ -27,27 +33,8 @@ from lintian_brush.rules import (
     )
 
 
-compat_release = os.environ.get('COMPAT_RELEASE', 'sid')
-
-new_debhelper_compat_version = {
-    # Debian
-    'jessie': 9,
-    'stretch': 10,
-    'sid': 12,
-    'bullseye': 12,
-    'buster': 12,
-
-    # Ubuntu
-    'xenial': 9,
-    'bionic': 11,
-    'cosmic': 11,
-    'disco': 12,
-    'eoan': 12,
-    }.get(compat_release)
-
-if new_debhelper_compat_version is None:
-    new_debhelper_compat_version = lowest_non_deprecated_compat_level()
-
+new_debhelper_compat_version = maximum_debhelper_compat_version(
+    compat_release())
 
 uses_cdbs = check_cdbs()
 
@@ -103,26 +90,23 @@ if os.path.exists('debian/compat'):
 
     current_debhelper_compat_version = read_debian_compat_file('debian/compat')
     if current_debhelper_compat_version < new_debhelper_compat_version:
-        with open('debian/compat', 'w') as f:
-            f.write('%s\n' % new_debhelper_compat_version)
+        with open('debian/compat', 'w') as cf:
+            cf.write('%s\n' % new_debhelper_compat_version)
     else:
         # Nothing to do
         sys.exit(2)
 
-    def bump_debhelper(control):
-        control["Build-Depends"] = ensure_minimum_version(
-                control.get("Build-Depends", ""),
-                "debhelper",
-                "%d~" % new_debhelper_compat_version)
-
-    update_control(source_package_cb=bump_debhelper)
+    with ControlUpdater() as updater:
+        updater.source["Build-Depends"] = ensure_minimum_version(
+            updater.source.get("Build-Depends", ""),
+            "debhelper",
+            "%d~" % new_debhelper_compat_version)
 else:
     # Assume that the compat version is set in Build-Depends
-    def bump_debhelper_compat(control):
-        global current_debhelper_compat_version
+    with ControlUpdater() as updater:
         try:
             offset, debhelper_compat_relation = get_relation(
-                control.get("Build-Depends", ""), "debhelper-compat")
+                updater.source.get("Build-Depends", ""), "debhelper-compat")
         except KeyError:
             sys.exit(2)
         else:
@@ -135,13 +119,11 @@ else:
             current_debhelper_compat_version = int(
                 debhelper_compat_relation[0].version[1])
         if current_debhelper_compat_version < new_debhelper_compat_version:
-            control["Build-Depends"] = ensure_exact_version(
-                    control["Build-Depends"],
+            updater.source["Build-Depends"] = ensure_exact_version(
+                    updater.source["Build-Depends"],
                     "debhelper-compat",
                     "%d" % new_debhelper_compat_version)
 
-    update_control(source_package_cb=bump_debhelper_compat)
-
 
 # For a list of behavior changes between debhelper compat verisons, see
 # https://manpages.debian.org/testing/debhelper/debhelper.7.en.html#Supported_compatibility_levels
@@ -365,6 +347,23 @@ def upgrade_to_installsystemd(line, target):
 def rename_installsystemd_target(rule):
     rule.rename_target(
         b'override_dh_systemd_enable', b'override_dh_installsystemd')
+    rule.rename_target(
+        b'override_dh_systemd_start', b'override_dh_installsystemd')
+
+
+def upgrade_to_debhelper_10():
+
+    # dh_installinit will no longer install a file named debian/package as an
+    # init script.
+
+    with ControlUpdater() as updater:
+        for binary in updater.binaries:
+            name = binary['Package']
+            if os.path.isfile(os.path.join('debian', name)):
+                os.rename(os.path.join('debian', name),
+                          os.path.join('debian', name + '.init'))
+                subitems.add(
+                    'Rename debian/%s to debian/%s.init.' % (name, name))
 
 
 def upgrade_to_debhelper_11():
@@ -384,12 +383,69 @@ def upgrade_to_debhelper_11():
         subitems.add('Drop obsolete upstart file %s.' % name)
         with open('debian/%s.maintscript' % package, 'a') as f:
             f.write('rm_conffile /etc/init/%s.conf %s\n' % (
-                service, os.environ['CURRENT_VERSION']))
+                service, str(current_package_version())))
+
+
+def drop_dh_missing_fail(line, target):
+    """Remove dh_missing --fail-missing."""
+    # Note that this is not currently replacing plain calls to dh_missing with
+    # empty an "override_dh_missing" rule.
+
+    if not line.startswith(b'dh_missing '):
+        return line
+    argv = line.split(b' ')
+    for arg in [b'--fail-missing', b'-O--fail-missing']:
+        if arg in argv:
+            # If we added the --fail-missing argument first, then
+            # don't announce that here.
+            try:
+                subitems.remove(
+                    'debian/rules: Move --fail-missing argument '
+                    'to dh_missing.')
+            except KeyError:
+                subitems.add(
+                    'debian/rules: Drop --fail-missing argument '
+                    'to dh_missing, which is now the default.')
+            else:
+                subitems.add(
+                    'debian/rules: Drop --fail-missing argument, '
+                    'now the default.')
+            argv.remove(arg)
+    return b' '.join(argv)
+
+
+def upgrade_to_debhelper_13():
+
+    # dh_installtempfiles will handle d/tmpfile{,s}.  It prefer the latter
+    # and warns about the former.
+
+    with ControlUpdater() as updater:
+        for binary in updater.binaries:
+            name = binary['Package']
+            tmpfile = os.path.join('debian', name + ".tmpfile")
+            if os.path.isfile(tmpfile):
+                os.rename(tmpfile, tmpfile + 's')
+                subitems.add(
+                    'Rename %s to %ss.' % (tmpfile, tmpfile))
+        if os.path.isfile(os.path.join('debian', "tmpfile")):
+            tmpfile = os.path.join('debian', "tmpfile")
+            os.rename(tmpfile, tmpfile + 's')
+            subitems.add(
+                'Rename %s to %ss.' % (tmpfile, tmpfile))
+
+    update_rules([
+        # The dh_missing command will now default to --fail-missing.  This can
+        # be reverted to a non-fatal warning by explicitly passing
+        # --list-missing like it was in compat 12.
+        drop_dh_missing_fail,
+        ])
 
 
 upgrade_to_debhelper = {
+    10: upgrade_to_debhelper_10,
     11: upgrade_to_debhelper_11,
     12: upgrade_to_debhelper_12,
+    13: upgrade_to_debhelper_13,
 }
 
 
@@ -407,9 +463,11 @@ if new_debhelper_compat_version > current_debhelper_compat_version:
     else:
         kind = "old"
         tag = "package-uses-old-debhelper-compat-version"
-    print("Bump debhelper from %s %s to %s." % (
-        kind, current_debhelper_compat_version, new_debhelper_compat_version))
+    lines = ["Bump debhelper from %s %s to %s." % (
+        kind, current_debhelper_compat_version, new_debhelper_compat_version)]
     for subitem in sorted(subitems):
-        print("+ " + subitem)
+        lines.append("+ " + subitem)
 
-    print("Fixed-Lintian-Tags: %s" % tag)
+    report_result(
+        description='\n'.join(lines),
+        fixed_lintian_tags=[tag])
