@@ -30,11 +30,11 @@ locale.setlocale(locale.LC_ALL, '')
 # Use better default than ascii with posix filesystems that deal in bytes
 # natively even when the C locale or no locale at all is given. Note that
 # we need an immortal string for the hack, hence the lack of a hyphen.
-sys._brz_default_fs_enc = "utf8"
+sys._brz_default_fs_enc = "utf8"   # type: ignore
 
 import breezy  # noqa: E402
-from breezy.errors import (
-    DependencyNotPresent,  # noqa: E402
+from breezy.errors import (  # noqa: E402
+    DependencyNotPresent,
     NotBranchError,
     )
 breezy.initialize()
@@ -122,6 +122,12 @@ def main(argv=None):
     parser.add_argument(
         '--exclude', metavar='EXCLUDE', type=str, action='append',
         help='Exclude a fixer.')
+    parser.add_argument(
+        '--opinionated', action='store_true',
+        help=argparse.SUPPRESS)
+    parser.add_argument(
+        '--diligent', action='count', default=0, dest='diligence',
+        help=argparse.SUPPRESS)
     parser.add_argument(
         'fixers', metavar='FIXER', nargs='*',
         help='specific fixer to run')
@@ -192,9 +198,6 @@ def main(argv=None):
                 minimum_certainty = cfg.minimum_certainty()
             if compat_release is None:
                 compat_release = cfg.compat_release()
-                if compat_release:
-                    compat_release = debian_info.codename(
-                        compat_release, default=compat_release)
             if allow_reformatting is None:
                 allow_reformatting = cfg.allow_reformatting()
             if update_changelog is None:
@@ -217,7 +220,9 @@ def main(argv=None):
                     allow_reformatting=allow_reformatting,
                     use_inotify=(False if args.disable_inotify else None),
                     subpath=subpath,
-                    net_access=not args.disable_net_access)
+                    net_access=not args.disable_net_access,
+                    opinionated=args.opinionated,
+                    diligence=args.diligence)
             except NotDebianPackage:
                 note("%s: Not a debian package.", wt.basedir)
                 return 1
