@@ -17,6 +17,13 @@
 
 """Helper functions for fixers."""
 
+from . import (
+    DEFAULT_MINIMUM_CERTAINTY,
+    certainty_sufficient,
+    )
+
+
+from debian.changelog import Version
 import os
 
 
@@ -42,3 +49,29 @@ def report_result(description, fixed_lintian_tags=None, certainty=None,
 def net_access_allowed():
     """Check whether network access is allowed."""
     return os.environ.get('NET_ACCESS', 'disallow') == 'allow'
+
+
+def compat_release():
+    return os.environ.get('COMPAT_RELEASE', 'sid')
+
+
+def current_package_version():
+    return Version(os.environ['CURRENT_VERSION'])
+
+
+def package_is_native():
+    return (not current_package_version().debian_revision)
+
+
+def meets_minimum_certainty(certainty):
+    return certainty_sufficient(
+        certainty,
+        os.environ.get('MINIMUM_CERTAINTY', DEFAULT_MINIMUM_CERTAINTY))
+
+
+def trust_package():
+    return os.environ.get('TRUST_PACKAGE') == 'true'
+
+
+def opinionated():
+    return os.environ.get('OPINIONATED', 'no') == 'yes'
