@@ -26,6 +26,7 @@ def test_suite():
         'copyright',
         'deb822',
         'debhelper',
+        'detect_gbp_dch',
         'dirty_tracker',
         'lintian_overrides',
         'multiarch_hints',
@@ -38,6 +39,7 @@ def test_suite():
         'upstream_metadata',
         'vcs',
         'watch',
+        'xdg',
         'yaml',
         ]
     module_names = [__name__ + '.test_' + name for name in names]
